/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2003 Julian Missig
 */

#include "AddContactDlg.hh"

#include "JabberConnection.hh"

#include <jabberoo/vCard.h>

#include <gtkmm/dialog.h>

using namespace Gabber;

void AddContactDlg::display(JabberConnection& conn, Gtk::Window& parent,
			     const Glib::ustring& jid,
			     const Glib::ustring& imsys)
{
     AddContactDlg* e = manage(new AddContactDlg(conn, parent, jid));
}

AddContactDlg::AddContactDlg(JabberConnection& conn, Gtk::Window& parent,
			     const Glib::ustring& jid,
			     const Glib::ustring& imsys)
     : BaseGabberWindow("AddContact_dlg"), _conn(conn)
{
     Gtk::Dialog* dlg  = static_cast<Gtk::Dialog*>(getGtkWindow());
     dlg->set_transient_for(parent);
     dlg->signal_response().connect(SigC::slot(*this, &AddContactDlg::on_dlg_response));

     get_widget("JID_ent", _entJID);
     _entJID->signal_changed().connect(SigC::slot(*this, &AddContactDlg::on_JID_ent_changed));
     _entJID->set_text(jid);

     get_widget("DisplayName_optmenu", _optmenuDisplayName);
     _optmenuDisplayName->signal_changed().connect(SigC::slot(*this, &AddContactDlg::on_DisplayName_optmenu_changed));

     get_widget("DisplayName_ent", _entDisplayName);

     get_widget("AddContact_btn", _btnAddContact);

     show();
}

void AddContactDlg::on_dlg_response(int resp)
{
     if (resp == Gtk::RESPONSE_CANCEL)
     {
	  close();
	  return;
     }

     _conn.getSession().roster() << jabberoo::Roster::Item(_entDisplayName->get_text(), _jid);

     _conn.getSession() << jabberoo::Presence(_jid, jabberoo::Presence::ptSubRequest);

     close();
}

void AddContactDlg::on_DisplayName_optmenu_changed()
{
     if (_optmenuDisplayName->get_history() == 0) // they just clicked "Choose..."
	  return;

     // This is the first time we've done this, so fetch the vCard
     if (!_entDisplayName->is_visible())
     {
	  Gtk::MenuItem* mi;
	  get_widget("Choose_menuitem", mi);
	  mi->hide();

	  // display fetching label
	  Gtk::Label* l;
	  get_widget("Fetching_lbl", l);
	  l->show();

	  // if we're doing this for a Gateway,
	  // then here is where we do an iq:transport JID lookup

	  // otherwise, the JID is just what's in the entry
	  _jid = _entJID->get_text();

	  // fetch the vcard
	  send_vcard_request();
     }
     else
     {
	  if (_optmenuDisplayName->get_history() == 1)
	  {
	       _conn_changed.disconnect();
	       _entDisplayName->set_text(_Fullname);
	       if (!_Fullname.empty())
		    _btnAddContact->set_sensitive(true);
	       _conn_changed = _entDisplayName->signal_changed().connect(SigC::slot(*this, &AddContactDlg::on_DisplayName_ent_changed));
	  }
	  else if (_optmenuDisplayName->get_history() == 2)
	  {
	       _conn_changed.disconnect();
	       _entDisplayName->set_text(_Nickname);
	       if (!_Nickname.empty())
		    _btnAddContact->set_sensitive(true);
	       _conn_changed = _entDisplayName->signal_changed().connect(SigC::slot(*this, &AddContactDlg::on_DisplayName_ent_changed));
	  }
     }
}

void AddContactDlg::send_vcard_request()
{
     // Get next session ID
     string id = _conn.getSession().getNextID();

     // Construct vCard request
     jabberoo::Packet iq("iq");
     iq.setID(id);
     iq.setTo(_jid);
     iq.getBaseElement().putAttrib("type", "get");
     judo::Element* vCard = iq.getBaseElement().addElement("vCard");
     vCard->putAttrib("xmlns", "vcard-temp");
     vCard->putAttrib("version", "2.0");
     vCard->putAttrib("prodid", "-//HandGen//NONSGML vGen v1.0//EN");

     // Send the vCard request
     _conn.getSession() << iq;
     _conn.getSession().registerIQ(id, slot(*this, &AddContactDlg::parse_vcard));
}

void AddContactDlg::parse_vcard(const judo::Element& t)
{
     // We should check for errors and report them!

     if (!t.empty())
     {
	  const judo::Element* vCardElem = t.findElement("vCard");
	  if (vCardElem != NULL)
	  {
	       jabberoo::vCard vc(*vCardElem);

	       _Nickname = vc[jabberoo::vCard::Nickname];
	       _Fullname = vc[jabberoo::vCard::Fullname];
	  }
     }

     if (_optmenuDisplayName->get_history() == 1)
     {
	  _entDisplayName->set_text(_Fullname);
	  if (!_Fullname.empty())
	       _btnAddContact->set_sensitive(true);
     }
     else if (_optmenuDisplayName->get_history() == 2)
     {
	  _entDisplayName->set_text(_Nickname);
	  if (!_Nickname.empty())
	       _btnAddContact->set_sensitive(true);
     }

     _conn_changed = _entDisplayName->signal_changed().connect(SigC::slot(*this, &AddContactDlg::on_DisplayName_ent_changed));


     Gtk::Label* l;
     get_widget("Fetching_lbl", l);
     l->hide();
     _entDisplayName->show();
}

void AddContactDlg::on_DisplayName_ent_changed()
{
     if (_optmenuDisplayName->get_history() != 3)
	  _optmenuDisplayName->set_history(3);

     if (_entDisplayName->get_text_length() > 0)
	  _btnAddContact->set_sensitive(true);
     else
	  _btnAddContact->set_sensitive(false);
}

void AddContactDlg::on_JID_ent_changed()
{
     // if they changed the JabberID.... we reset it all
     if (_entDisplayName->is_visible())
     {
	  _btnAddContact->set_sensitive(false);

	  if (_conn_changed.connected())
	       _conn_changed.disconnect();
	  _entDisplayName->hide();
	  _entDisplayName->set_text(Glib::ustring());

	  _Nickname = Glib::ustring();
	  _Fullname = Glib::ustring();

	  Gtk::MenuItem* mi;
	  get_widget("Choose_menuitem", mi);
	  mi->show();

	  _optmenuDisplayName->set_history(0);
     }
     else if (_entJID->get_text_length() <= 0)
     {
	  _optmenuDisplayName->set_sensitive(false);
     }
     else
     {
	  _optmenuDisplayName->set_sensitive(true);
     }
}
