/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#include "GabberApp.hh"

#include <config.h>

#include "Environment.hh"
#include "ConfigPaths.hh"
#include "TCPTransmitter.hh"
#include "PrefsInterface.hh" // for LoginDlg
#include "PluginManager.hh"
#include "S10nRequestDlg.hh"

#include <jabberoo/jabberoo.hh>

#include <gtkmm/messagedialog.h>

namespace Gabber {

// -------------------------------------------------------------------
//
// Ye Olde Gabber App - The Hub
//
// -------------------------------------------------------------------
GabberApp::GabberApp(int argc, char** argv)
     : _GtkMain(argc, argv)
{
     cout << ENV_VARS.package << " " << ENV_VARS.version << endl;

     _log_manager = new LogManager();

     // Instiatiate objects
     _resource_manager = new ResourceManager();
     P_Man = new PluginManager();

     // Filter for agents
     _agent_filter = new jabberoo::DiscoDB::DiscoFilter(AgentFilter(), _agents);
     _Session->discoDB().addFilter(_agent_filter);

     // Session signals
     _Session->evtTransmitXML.connect(slot(*this, &GabberApp::on_XML_transmit));
     _Session->evtRecvXML.connect(slot(*this, &GabberApp::on_XML_recv));
//     _Session->evtIQ.connect(slot(*this, &GabberApp::on_session_iq));
//     _Session->evtAuthError.connect(slot(*this, &GabberApp::on_session_auth_error));
     _Session->evtPresenceRequest.connect(slot(*this, &GabberApp::on_presence_request));
//     _Session->evtUnknownPacket.connect(slot(*this, &GabberApp::on_session_unknown_packet));
//     _Session->evtPresence.connect(slot(*this, &GabberApp::on_session_presence));
//     _Session->evtMyPresence.connect(slot(*this, &GabberApp::on_session_my_presence));

     // Check if we should do first-time wizard
     // if (config ...
     // otherwise, do they want to autoconnect?
     // else if (config ...
//     login();
     // ok, just give them the login dialog
     LoginDlg::execute();
}

GabberApp::~GabberApp()
{
     _Session->discoDB().removeFilter(_agent_filter);
     delete _agent_filter;
     delete _winMain;
     delete _resource_manager;
     delete _log_manager;
}

void GabberApp::run()
{
     // Let the gnome app start processing messages
     Gtk::Main::run();
}

void GabberApp::init_win()
{
    // If we don't have an existing main window
    if (_winMain == NULL)
    {
        // Create one
        _winMain  = new GabberWin(*this);
    }
    P_Man->loadAll();
}

void GabberApp::clean_disconnect()
{
     // Send out status
     *_Session << jabberoo::Presence("", jabberoo::Presence::ptUnavailable,
				     jabberoo::Presence::stOffline, _("Logged Out"));
     
     // Try disconnecting Session.. and if that fails, the Transmitter
     if (!_Session->disconnect())
	  _Transmitter->disconnect();
}

void GabberApp::quit()
{
     clean_disconnect();

     // Stop gtk/gnome message loop
     Gtk::Main::quit();
}

// -------------------------------------
// Session event handlers
// -------------------------------------
void GabberApp::on_XML_transmit(const char* XML)
{
#ifdef SESSION_DEBUG
    LogManager::getSingleton().debug() << jutil::getTimeStamp() << " send " 
        << XML << endl;
#endif // SESSION_DEBUG
}

void GabberApp::on_XML_recv(const char* XML)
{
#ifdef SESSION_DEBUG
    LogManager::getSingleton().debug() << jutil::getTimeStamp() << " recv " 
        << XML << endl;
#endif // SESSION_DEBUG
}

void GabberApp::on_presence_request(const jabberoo::Presence& pres)
{
    std::string jid = pres.getFrom();
    std::string reason = pres.getStatus();
    jabberoo::Presence::Type type = pres.getType();

    if (type == jabberoo::Presence::ptSubRequest)
    {
        S10nRequestDlg::display(*this, *_winMain->getGtkWindow(), jid, reason);
    }
    else if (type == jabberoo::Presence::ptUnsubRequest)
    {
        jabberoo::Presence p(jid, jabberoo::Presence::ptUnsubscribed);
        *_Session << p;
    }
}

} // namespace Gabber

