/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#ifndef INCL_GABBER_WIN_HH
#define INCL_GABBER_WIN_HH

#include "GabberApp.hh"
#include "BaseGabberWindow.hh"
#include "RosterView.hh"

#include <jabberoo.hh>

#include <gtkmm/optionmenu.h>
#include <gtkmm/menu.h>
#include <gtkmm/textview.h>

namespace Gabber
{

class GabberApp;

/**
 * The main Gabber window.
 */
class GabberWin : public BaseGabberWindow
{
public:
     /**
      * Create the Gabber window.
      * This should be done by GabberApp
      * @param app The GabberApp instance we are creating this window for.
      */
     GabberWin(GabberApp& app);
     ~GabberWin();
protected:
     /**
      * Connect all the signals from the menu items.
      * Should only be called once by this GabberWin.
      */
     void init_menus();
     /**
      * Close the GabberWin, quitting the GabberApp as well.
      */
     void close();

     // Callbacks
     void on_evtConnecting();
     void on_evtConnected();
     void on_discoDB_cache(const jabberoo::DiscoDB::Item* item);
     void on_evtDisconnected();
     void on_my_presence_event(const jabberoo::Presence& p);
     // Menu items
     void on_logout_item_activate();
     void on_add_contacts_activate();
     void on_browse_agents_activate();
     void on_joingc_activate();
     void on_roster_hideoffline_activate();

    // Widget Callbacks
     void on_pres_menu_changed();
     bool on_status_txt_event(GdkEvent* e);
     bool on_status_txt_unfocused(GdkEventFocus* e);
     void set_status_from_status_txt();
private:
     GabberApp&           _app;
     Glib::RefPtr<RosterModel>  _rostermodel;
     RosterView*          _rosterview;
     Gtk::OptionMenu*     _optPres;
     int                  _prev_show;
     Gtk::ScrolledWindow* _scrollStatus;
     Gtk::TextView*       _txtStatus;
     Gtk::Label*          _lblStatus;

}; // class GabberWin

}; // namespace Gabber

#endif // INCL_GABBER_WIN_HH
