/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2003 Julian Missig
 */

#include "S10nRequestDlg.hh"

#include "GabberUtility.hh"

#include <jabberoo/jabberoo.hh>

#include <gtkmm/dialog.h>
#include <gtkmm/label.h>
#include <gtkmm/textview.h>

using namespace Gabber;

void S10nRequestDlg::display(JabberConnection& app, Gtk::Window& parent, 
                             const Glib::ustring& jid,
                             const Glib::ustring& reason)
{
    S10nRequestDlg* dlg = manage(new S10nRequestDlg(app, parent, jid, reason));
}

S10nRequestDlg::S10nRequestDlg(JabberConnection& app, Gtk::Window& parent, 
                               const Glib::ustring& jid, 
                               const Glib::ustring& reason) : 
    BaseGabberWindow("S10nRequest_dlg"), _app(app), _jid(jid)
{ 
    Gtk::Label* lbl;
    get_widget("JID_lbl", lbl);
    lbl->set_text(jid);

    get_widget("S10n_lbl", lbl);
    lbl->set_markup(Util::substitute("<b>%s</b> would like to add you to their Contact List.", _jid));

    Gtk::TextView* tv;
    get_widget("RequestReason_txtview", tv);
    tv->get_buffer()->set_text(reason);

    Gtk::Dialog* dlg  = static_cast<Gtk::Dialog*>(getGtkWindow());
    dlg->set_transient_for(parent);
    dlg->signal_response().connect(SigC::slot(*this, 
                                            &S10nRequestDlg::on_dlg_response));

    show();
}

S10nRequestDlg::~S10nRequestDlg()
{ }

void S10nRequestDlg::on_dlg_response(int resp)
{
    hide();
    
    jabberoo::Presence p(_jid, jabberoo::Presence::ptSubscribed);

    // Approve them
    if (resp == Gtk::RESPONSE_REJECT)
        p.setType(jabberoo::Presence::ptUnsubscribed);

    _app.getSession() << p;

    // See if we should subscribe back
    Gtk::ToggleButton* btn;
    get_widget("AddToList_chk", btn);
    if (btn->get_active() && resp == Gtk::RESPONSE_ACCEPT)
    {
        p.setType(jabberoo::Presence::ptSubRequest);
        _app.getSession() << p;
    }

    close();
}
