/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#ifndef INCL_CHAT_VIEW_HH
#define INCL_CHAT_VIEW_HH

#include "BaseGabberWindow.hh"
#include "PlainTextView.hh"

#include <string>
#include <map>
#include <jabberoo/jabberoo.hh>
#include <gtkmm/button.h>
#include <gtkmm/textview.h>
#include <gtkmm/togglebutton.h>
#include <gtkmm/menuitem.h>

using namespace jabberoo;

namespace Gabber {

class ChatViewManager;
class PrettyJID;

class ChatView
     : public BaseGabberWindow
{
public:
     ChatView(ChatViewManager& mgr, const judo::Element& msg);
     ChatView(ChatViewManager& mgr, const std::string& jid);
     ~ChatView();
     
     void close();
     
protected:
     void init(bool is_blank);
     // Handlers
     void on_Send_clicked();
//     void on_CloseChat_Close_clicked();
//     void on_CloseChat_Keep_clicked();
     bool on_delete_event(GdkEventAny* e);
     bool on_window_event(GdkEvent* e);
     void on_AddContact_clicked();
     void on_ViewContact_clicked();
     void on_History_clicked();
     // Presence handler
//     void on_presence(const Presence& p, const Presence::Type prev);
     // Error handler
//     void on_error(const string& concerning, int errorcode, const string& errormsg, const string& body);
//     void on_Chatview_drag_data_received(GdkDragContext* drag_ctx, gint x, gint y, GtkSelectionData* data, guint info, guint time);
     
     /// XPath Callbacks
     void on_chat_node(const Element& msg);
     void on_event_node(const Element& msg);
     void on_presence_node(const Element& e);

     void on_resource_changed();
     void on_resource_locked();

private:
     ChatViewManager&       _mgr;
     bool                   _onRoster;
     Glib::ustring          _jid;
     Glib::ustring          _thread;
     Glib::ustring          _local_nick;
     Glib::ustring          _nickname;
     time_t                 _last_msg_time;

     Gtk::ToggleButton*     _tglbtnSendAsMsg;
     Gtk::Button*           _btnAddContact;
     PrettyText*            _txtChatview;
     Gtk::TextView*         _txtMessage;
     PrettyJID*             _prettyjid;

     judo::XPath::Query*    _chat_query;
     judo::XPath::Query*    _event_query;
     judo::XPath::Query*    _presence_query;

     bool                   _sent_composing;
     std::string            _composing_id;

     std::list<Message*>    _pending_msgs;

     void display(const Message& m);
};

}; // namespace Gabber
#endif // INCL_CHAT_VIEW_HH
