/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#include "StandaloneView.hh"

#include "StandaloneSendDlg.hh"
#include "StandaloneViewManager.hh"
#include "GabberApp.hh"
#include "GabberUtility.hh"
#include "GabberWidgets.hh"

using namespace Gabber;

StandaloneView::StandaloneView(StandaloneViewManager& mgr, 
                               const judo::Element& msg) : 
    BaseGabberWindow("StandaloneMsgRecv_dlg"), _mgr(mgr),
    _jid(msg.getAttrib("from"))
{
    // Add the PrettyJID
    Gtk::HBox* hbox;
    get_widget("JIDInfo_hbox", hbox);
    PrettyJID* pj = Gtk::manage(new PrettyJID(_jid, "", PrettyJID::dtNickJID));
    hbox->pack_end(*pj);
    getGtkWindow()->set_title(
        Util::substitute(_("Message from %s"), pj->get_nickname()));
    pj->show();

    // Disable the buttons
    get_widget("Next_btn", _next_button);
    _next_button->set_sensitive(false);
    _next_button->signal_clicked().connect(
        SigC::slot(*this, &StandaloneView::on_next_clicked));
    get_widget("Previous_btn", _prev_button);
    _prev_button->set_sensitive(false);
    _prev_button->signal_clicked().connect(
        SigC::slot(*this, &StandaloneView::on_prev_clicked));

    // Hook up our other buttons
    Gtk::Button* btn;
    get_widget("Reply_btn", btn);
    btn->signal_clicked().connect(
        SigC::slot(*this, &StandaloneView::on_reply_clicked));
    get_widget("Close_btn", btn);
    btn->signal_clicked().connect(
        SigC::slot(*this, &StandaloneView::on_close_clicked));

    // Pull out the other main widgets
    get_widget("Subject_lbl", _subject_lbl);
    get_widget("Subject_lbl_lbl", _subject_lbl_lbl);
    get_widget("Time_lbl", _time_lbl);
    get_widget("Message_txtview", _message_txtview);

    // Add the msg and then display it
    add_message(msg);
    _cur_msg = _msgs.begin();
    display();
}

StandaloneView::~StandaloneView()
{
    for_each(_msgs.begin(), _msgs.end(), Util::ListDeleter());
}

void StandaloneView::close()
{
    _mgr.releaseView(_jid);

    BaseGabberWindow::close();
}

void StandaloneView::add_message(const judo::Element& msg)
{
    _msgs.push_back(new jabberoo::Message(msg));

    if (_msgs.size() > 1)
    {
        // We can now go forward...
        _next_button->set_sensitive(true);
    }
}

void StandaloneView::display()
{
    jabberoo::Message* msg = *_cur_msg;

    Glib::ustring subject = msg->getSubject();
    if (subject.empty())
    {
        _subject_lbl->hide();
        _subject_lbl_lbl->hide();
    }
    else
    {
        _subject_lbl->set_text(msg->getSubject());
        _subject_lbl->show();
        _subject_lbl_lbl->show();
    }
    _time_lbl->set_text(msg->getDateTime());
    _message_txtview->get_buffer()->set_text(msg->getBody());
}

void StandaloneView::on_close_clicked()
{
    close();
}

void StandaloneView::on_reply_clicked()
{
     jabberoo::Message* msg = *_cur_msg;
     StandaloneSendDlg::display(G_App, *_thisWindow, *msg);
}

void StandaloneView::on_prev_clicked()
{
    --_cur_msg;
    if (_cur_msg == _msgs.begin())
    {
        _prev_button->set_sensitive(true);
    }

    _next_button->set_sensitive(true);

    display();
}

void StandaloneView::on_next_clicked()
{
    ViewMsgMap::iterator next = ++_cur_msg;
    ++next;
    if (next == _msgs.end())
    {
        _next_button->set_sensitive(false);
    }

    _prev_button->set_sensitive(true);

    display();
}
