/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#include "StandaloneViewManager.hh"

#include "GabberApp.hh"
#include "StandaloneSendDlg.hh"

#include <sigc++/slot.h>

using namespace Gabber;

static StandaloneViewManager* svm_instance;

extern "C" const gchar* g_module_check_init(GModule* module)
{
    svm_instance = new StandaloneViewManager();
    return NULL;
}

extern "C" void g_module_unload(GModule* module)
{ delete svm_instance; }

StandaloneViewManager::StandaloneViewManager() :
    _menu_item("Standalone Message...", true)
{
    _xp_query = GabberApp::getSingleton().getSession().registerXPath(
        "/message[not(@type) or @type='normal']", slot(*this,
        &StandaloneViewManager::on_message_node));
    _menu_item.signal_activate().connect(SigC::slot(*this, 
        &StandaloneViewManager::on_menu_item_activate));
    Popups::User::getSingleton().addItem(&_menu_item);
}

StandaloneViewManager::~StandaloneViewManager()
{
    GabberApp::getSingleton().getSession().unregisterXPath(_xp_query);
    ViewMap::iterator it = _views.begin();
    while (it != _views.end())
    {
        ViewMap::iterator next = it;
        ++next;

        delete (*it).second;

        it = next;
    }
}

void StandaloneViewManager::releaseView(const std::string& jid)
{
    _views.erase(jabberoo::JID::getUserHost(jid));
}

void StandaloneViewManager::on_menu_item_activate()
{
     Glib::ustring to = Popups::User::getSingleton().getSelectedJID();
     StandaloneSendDlg::display(G_App, to);
}

void StandaloneViewManager::on_message_node(const Element& elem)
{
    // This isn't a valid node
    if (elem.findElement("body") == NULL)
    {
        return;
    }

    std::string from = jabberoo::JID::getUserHost(elem.getAttrib("from"));
    ViewMap::iterator it = _views.find(from);
    if (it == _views.end())
    {
        _views.insert(ViewMap::value_type(from, 
            SigC::manage(new StandaloneView(*this, elem))));
    }
    else
    {
        it->second->add_message(elem);
    }
}

